@@ -3,14 +3,14 @@ package Hessian::Tiny::Client;
 use warnings;
 use strict;
 
-require 5.6.0;
+require 5.006;
 
 use URI ();
 use IO::File ();
-use File::Temp ();
 use LWP::UserAgent ();
 use HTTP::Headers ();
 use HTTP::Request ();
+use File::Temp ();
 
 use Hessian::Tiny::ConvertorV1 ();
 use Hessian::Tiny::ConvertorV2 ();
@@ -21,12 +21,12 @@ Hessian::Tiny::Client - Hessian RPC Client implementation in pure Perl
 
 =head1 VERSION
 
-Version 0.07
+Version 0.12
 
 =cut
 
-our $VERSION = '0.07';
-our $ErrStr;
+our $VERSION = '0.12';
+our $Error;
 
 
 =head1 SYNOPSIS
@@ -41,10 +41,18 @@ our $ErrStr;
     if($stat == 0){ # success
         print "2 + 4 = $res";
     }else{
-        print "error: $Hessian::Tiny::Client::ErrStr";
+        print "error: $Hessian::Tiny::Client::Error";
     }
-    ...
 
+=head1 DESCRIPTION
+
+Hessian is a compact binary protocol for web communication in form of client/server RPC.
+
+This module allows you to write Hessian clients in Perl.
+
+This module supports Hessian Protocol 1.0 and 2.0
+
+Perl 5.6.0 or later is required to install this modle.
 
 =head1 SUBROUTINES/METHODS
 
@@ -58,8 +66,30 @@ our $ErrStr;
         hessian_flag => 1, # if you need strong typing in return value
     );
 
-argument 'url' need to be a valid url, otherwise the constructor will return undef.
+=over
 
+=item 'url'
+
+hessian server url, need to be a valid url, otherwise the constructor will return undef.
+
+=item 'version'
+
+hessian protocol version, 1 or 2.
+
+=item 'debug'
+
+for debugging, you probably don't need to set this flag.
+
+=item 'auth'
+
+if http server requires authentication. (passed on to LWP request)
+
+=item 'hessian_flag'
+
+default off, that means return value are automatically converted into native perl data;
+if set to true, you will get Hessian::Type::* object as return.
+
+=back
 
 =cut
 
@@ -68,7 +98,7 @@ sub new {
   my $self = {@params};
   my $u = URI->new($self->{url});
   unless(defined $u and $u->scheme and $u->scheme =~ /^http/){
-    $ErrStr = qq[Hessian url not valid: '$$self{url}'];
+    $Error = qq[Hessian url not valid: '$$self{url}'];
     return;
   }
   $self->{version} ||= 1; #default v1.0
@@ -109,23 +139,24 @@ sub new {
         print "error: $res";
     }
 
-return value, $stat: 0 for success, 1 for Fault, 2 for communication errors;
-$res will hold error (Hessian::Fault or string) in case of unsuccessful call;
-$res will hold return value in case of successful call; normally Hessian types
-are converted to perl data directly, if you want strong typing in return value,
-you can set (hessian_flag => 1) in the constructor call new().
+=over
+
+=item return values:
+
+B<$stat>: 0 for success, 1 for Hessian level Fault, 2 for other errors such as http communication error or parsing anomaly;
+
+B<$res>: will hold the hessian call result if call was successful, or will hold error (Hessian::Fault or string) in case of unsuccessful call;
+
+normally Hessian types are converted to perl data directly, if you want strong typing in return value, you can set (hessian_flag => 1) in the constructor call new().
 
 =cut
 
 sub call {
   my($self,$method_name,@hessian_params) = @_;
 
-  $ErrStr = ''; # reset, probably not needed
+  $Error = ''; # reset, probably not needed
 # open fh to write call
-  my $call_fh = File::Temp->new(
-      UNLINK => ($self->{debug} ? 0 : 1),
-      SUFFIX => '.htc.dat'
-  );
+  my $call_fh = File::Temp::tempfile();
   return 2, $self->_elog("call, open temp call file failed $!") unless defined $call_fh;
 
 # write call to fh
@@ -138,54 +169,58 @@ sub call {
     }
     1;
   }or return 2, $self->_elog("write_call: $@");
-  $self->_elog("call: written (@{[$call_fh->filename]})");
 
 # write call successful, rewind & read
-  my $call_fn = $call_fh->filename;
-  $call_fh->close;
-  $call_fh = IO::File->new($call_fn);
+  $call_fh->flush();
+  seek $call_fh,0,0;
 
 # make LWP client
   my $ua = LWP::UserAgent->new;
   $ua->agent("Perl Hessian::Tiny::Client $$self{version}");
   my $header = HTTP::Headers->new();
 
-  if('ARRAY' eq ref $self->{auth}){
+  if('ARRAY' eq ref $self->{auth} and
+    length $self->{auth}->[0] > 0 and
+    length $self->{auth}->[1] > 0 
+  ){
     $header->authorization;
     $header->authorization_basic($self->{auth}->[0],$self->{auth}->[1]);
   }
-  my $http_request = HTTP::Request->new(POST => $self->{url}, $header);
   my $buf = '';
-  $http_request->add_content($buf) while( 0 < read($call_fh,$buf,255));
-  $call_fh->close;
-  unlink $call_fn unless $self->{debug}; # remove file
+  binmode $call_fh,':bytes';
+  my $http_request = HTTP::Request->new(POST => $self->{url}, $header,sub{
+    read $call_fh,$buf,255; $buf
+  });
 
 # send http request
-  my $reply_fh = File::Temp->new(
-      UNLINK => ($self->{debug} ? 0 : 1),
-      SUFFIX => '.htr.dat'
-  );
+  my $reply_fh = File::Temp::tempfile();
   return 2, $self->_elog("call, open temp reply file failed $!") unless defined $reply_fh;
-  $reply_fh->close;
-  my $http_response = $ua->request($http_request, $reply_fh->filename);
-  $self->_elog("reply written to (@{[$reply_fh->filename]})");
+  binmode $reply_fh,':bytes';
+  my $http_response = $ua->request($http_request, sub{
+    my($chunk,$res,$lwp) = @_; print $reply_fh $chunk;
+  });
+  $call_fh->close;
 
-  if($http_response->is_success){
-    my($st,$re) = _read_reply( Hessian::Tiny::Type::_make_reader($reply_fh->filename),
-                               $self->{hessian_flag});
-    $self->_elog("Fault: $re->{code}; $re->{message}") if $st && 'Hessian::Type::Fault' eq ref $re;
-    $self->_elog($re) if $st == 2;
-    return $st,$re;
-  } # if http successful
-
-  unlink $reply_fh->filename unless $self->{debug};
-# http level failure
-  return 2, $self->_elog('Hessian http response unsuccessful: ',
-    $http_response->status_line, $http_response->error_as_HTML);
+  unless($http_response->is_success){ # http level failure
+    $reply_fh->close;
+    return 2, $self->_elog('Hessian http response unsuccessful: ',
+      $http_response->status_line, $http_response->error_as_HTML)
+    ;
+  }
 
+  my($st,$re);
+  $reply_fh->flush();
+  seek $reply_fh,0,0;
+  eval{
+    ($st,$re) = _read_reply( Hessian::Tiny::Type::_make_reader($reply_fh),$self->{hessian_flag});
+    1;
+  } or return 2, $self->_elog("Hessian parse reply: $@");
+  $self->_elog("Fault: $re->{code}; $re->{message}") if $st && 'Hessian::Type::Fault' eq ref $re;
+  $self->_elog($re) if $st == 2;
+  return $st,$re;
 }
 
-sub _elog { my $self=shift;$ErrStr=join'',@_;print STDERR @_,"\n" if $self->{debug}; join '',@_ }
+sub _elog { my $self=shift;$Error=join'',@_;print STDERR @_,"\n" if $self->{debug}; join '',@_ }
 sub _read_reply {
   my($reader,$hessian_flag) = @_;
   my $buf = $reader->(3);
@@ -224,6 +259,8 @@ sub _read_reply {
   }
 }
 
+=back
+
 =head1 HESSIAN DATA TYPES
 
 =head2 Null
@@ -265,6 +302,7 @@ when 'hessian_flag' is set to true, you will get Math::BigInt.
 
 As return value, by default, you will get the number directly;
 when 'hessian_flag' is set to true, you will get Hessian::Type::Double.
+Note, floating point numbers may appear slightly inaccurate, due to the binary nature of machines (not the fault of protocol itself, or Perl even).
 
 =head2 Date
 
@@ -282,7 +320,7 @@ when 'hessian_flag' is set to true, you will get Hessian::Type::Date (milli sec
 
 As return value, by default, you will get the perl string;
 when 'hessian_flag' is set to true, you will get Hessian::Type::Binary or
-Hessian::Type::String object.
+Hessian::Type::String object. (Binary means byte stream, while String is UTF-8)
 
 =head2 XML
 
@@ -348,7 +386,7 @@ You can find documentation for this module with the perldoc command.
     perldoc Hessian::Tiny::Client
 
 
-For information on the protocol itself, take a look at:
+For information on the wonderful protocol itself, take a look at:
 	http://hessian.caucho.com/
 
 =over 4
